objectives_data = [
    {
        "obj_id": '1',
        "primary": {
    "icon": 'local_offer',
    "title": 'sales',
    },
    "secondary": {
    "icon": 'local_offer',
    "title": 'sales',
    "description": 'sales_description',
    },
},
{
    "obj_id": '2',
    "primary": {
    "icon": 'people',
    "title": 'prospects',
    },
    "secondary": {
    "icon": 'people',
    "title": 'prospects',
    "description": 'prospect_description',
    },
},
{
    "obj_id": '3',
    "primary": {
    "icon": 'web',
    "title": 'traffic',
    },
    "secondary": {
    "icon": 'web',
    "title": 'traffic',
    "description": 'traffic_description',
    },
},
{
    "obj_id": '4',
    "primary": {
    "icon": 'auto_awesome',
    "title": 'brand_interest',
    },
    "secondary": {
    "icon": 'auto_awesome',
    "title": 'brand_interest',
    "description": 'brand_interest_description',
    },
},
{
    "obj_id": '5',
    "primary": {
    "icon": 'stars',
    "title": 'audience_brand_awareness',
    },
    "secondary": {
    "icon": 'stars',
    "title": 'audience_brand_awareness',
    "description": 'audience_brand_awareness_description',
    },
},
{
    "obj_id": '6',
    "primary": {
    "icon": 'settings',
    "title": 'no_objective',
    },
    "secondary": {
    "icon": 'settings',
    "title": 'no_objective',
    "description": 'unspecified_objective',
    },
}
]